
set echo on

CREATE OR REPLACE PACKAGE log_pkg AS

   TYPE log_handle_rt IS RECORD
   ( id    VARCHAR2(100)
   , sid   INTEGER
   , uname VARCHAR2(30) );

   FUNCTION start_log( 
            p_lvl IN INTEGER DEFAULT 1
            ) RETURN log_handle_rt;
 
   PROCEDURE log( 
             p_hdl IN log_handle_rt,
             p_msg IN VARCHAR2,
             p_lvl IN INTEGER DEFAULT 1
             );

   PROCEDURE end_log( 
             p_hdl IN log_handle_rt 
             );

   PROCEDURE set_logging_level(
             p_sid IN INTEGER,
             p_lvl IN INTEGER 
             );

   FUNCTION logging_level(
            p_sid IN INTEGER
            ) RETURN INTEGER;

END log_pkg;
/

CREATE OR REPLACE PACKAGE BODY log_pkg AS

   c_ctx CONSTANT VARCHAR2(10) := 'LOG_CTX';

   -------------------------------------------------------------

   FUNCTION get_caller RETURN VARCHAR2 IS
      v_stk VARCHAR2(4000) := DBMS_UTILITY.FORMAT_CALL_STACK;
      v_dpt PLS_INTEGER := 6;
      v_pos PLS_INTEGER := 21;
      v_dlm VARCHAR2(1) := CHR(10);
   BEGIN
      RETURN NVL(
                SUBSTR(
                   SUBSTR(
                      v_stk,
                      INSTR( v_stk, v_dlm ,1,(v_dpt-1))+1,
                      INSTR( v_stk, v_dlm ,1, v_dpt) - 
                         (INSTR( v_stk, v_dlm, 1, (v_dpt-1)))-1
                      ),
                   v_pos ),
                '[unknown]' );
   END get_caller;
   
   -------------------------------------------------------------

   FUNCTION get_sid RETURN INTEGER IS
      v_sid INTEGER;
   BEGIN
      SELECT sid INTO v_sid FROM v$mystat WHERE ROWNUM = 1;
      RETURN v_sid;
   END get_sid;

   ---------------------------------------------------------------

   FUNCTION get_handle(
            p_sid IN INTEGER
            ) RETURN log_handle_rt IS

      r_hdl log_handle_rt;

   BEGIN

      SELECT attribute, client_identifier, username INTO r_hdl
      FROM   global_context
      WHERE  client_identifier = TO_CHAR(p_sid);

      RETURN r_hdl;

   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         RAISE_APPLICATION_ERROR(
            -20000, 'SID not found in logging context'
            );
   END get_handle;

   ---------------------------------------------------------------

   PROCEDURE set_identifier( 
             p_sid IN INTEGER 
             ) IS
   BEGIN
      DBMS_SESSION.SET_IDENTIFIER(TO_CHAR(p_sid));
   END set_identifier;

   ---------------------------------------------------------------

   PROCEDURE clear_identifier IS
   BEGIN
      DBMS_SESSION.CLEAR_IDENTIFIER;
   END clear_identifier;

   ---------------------------------------------------------------

   PROCEDURE set_logging_level_internal(
             p_hdl IN log_handle_rt,
             p_lvl IN INTEGER 
             ) IS
   BEGIN
      DBMS_SESSION.SET_CONTEXT( 
         c_ctx, p_hdl.id, TO_CHAR(p_lvl), p_hdl.uname, 
         TO_CHAR(p_hdl.sid)
         );
   END set_logging_level_internal;

   ---------------------------------------------------------------

   FUNCTION start_log( 
            p_lvl IN INTEGER DEFAULT 1
            ) RETURN log_handle_rt IS

      v_caller VARCHAR2(100);
      r_handle log_handle_rt;

   BEGIN

      v_caller := get_caller();

      r_handle.id := SUBSTR(v_caller,INSTR(v_caller,'.')+1);
      r_handle.sid := get_sid();
      r_handle.uname := SUBSTR(
                           SUBSTR(v_caller,1,INSTR(v_caller,'.')-1),
                           INSTR(v_caller,' ',-1)+1
                           );

      set_identifier(r_handle.sid);
      set_logging_level_internal(r_handle, p_lvl);
      clear_identifier();

      RETURN r_handle;

   END start_log;
 
   ---------------------------------------------------------------
   
   FUNCTION logging_level( 
            p_hdl IN log_handle_rt
            ) RETURN INTEGER IS
   BEGIN
      RETURN TO_NUMBER(SYS_CONTEXT('log_ctx',p_hdl.id));
   END logging_level;

   ---------------------------------------------------------------

   FUNCTION logging_level( 
            p_sid IN INTEGER
            ) RETURN INTEGER IS
      r_hdl log_handle_rt;
      v_lvl INTEGER;
   BEGIN
      set_identifier(p_sid);
      r_hdl := get_handle(p_sid);
      v_lvl := logging_level(r_hdl);
      clear_identifier();
      RETURN v_lvl;
   END logging_level;

   ---------------------------------------------------------------

   PROCEDURE log( p_hdl IN log_handle_rt,
                  p_msg IN VARCHAR2,
                  p_lvl IN INTEGER DEFAULT 1 ) IS
   BEGIN
      set_identifier(p_hdl.sid);
      IF logging_level(p_hdl) >= p_lvl THEN
         DBMS_OUTPUT.PUT_LINE(
            TO_CHAR(SYSDATE,'HH24:MI:SS ": "') || p_msg
            );
      END IF;
      clear_identifier();
   END log;

   ---------------------------------------------------------------

   PROCEDURE end_log( 
             p_hdl IN log_handle_rt 
             ) IS
   BEGIN
      set_identifier(p_hdl.sid);
      DBMS_SESSION.CLEAR_CONTEXT( 
         c_ctx, TO_CHAR(p_hdl.sid), p_hdl.id
         );
      clear_identifier();
   END end_log;

   ---------------------------------------------------------------

   PROCEDURE set_logging_level(
             p_sid IN INTEGER,
             p_lvl IN INTEGER 
             ) IS
      r_hdl log_handle_rt;
   BEGIN
      set_identifier(p_sid);
      r_hdl := get_handle(p_sid);
      set_logging_level_internal(r_hdl, p_lvl);
      clear_identifier();
   END set_logging_level;

   ---------------------------------------------------------------

END log_pkg;
/

sho err
